"""
========================================
  Introduction to Computer Programming
========================================

----------------------------------------------------
  examples in preparation for lab 2:
    * ord/chr operations
    * functions that take more than one argument
    * simple Boolean expressions
    * simple if/else statements
    * very simple graphics
----------------------------------------------------
"""


from simple_graphics import *


def shift1(word):
    """Returns new string that is each character of s shifted one place.

    >>> shift1('HAL')
    'IBM'
    """
    shifted_word = ''
    for symbol in word:
        original_code = ord(symbol)
        new_code = original_code + 1
        new_symbol = chr(new_code)
        shifted_word = shifted_word + new_symbol
    return shifted_word


def crude_lowercase(s):
    """
    Returns new string that is like s but converted to lowercase.
    Assumes that s consists entirely of uppercase letters.

    >>> crude_lowercase('WORLD')
    'world'
    """
    t = ''
    for ch in s:
        t = t + (chr(ord(ch) + 32))
    return t


# like the quadratic formula; takes 4 numeric arguments
def quadratic(a, b, c, x):
    y = a*x*x + b*x + c
    return y


# a function that returns a Boolean value
def is_even(n):
    b = n % 2 == 0
    return b


# a canonical use of if/else
def absolute_value(x):
    if x < 0:
        y = -x
    else:
        y = x
    return y


# another canoncial use of if/else
def minimum(x, y):
    if x < y:
        z = x
    else:
        z = y
    return z


def greeting(name):
    if name != 'Mike':
        response = 'Nice to meet you!'
    else:
        response = 'You again?'
    return response


def ulam(m):
    if m % 2 == 0:
        n = m / 2
    else:
        n = 3*m + 1
    return n


def count_i(s):
    """Returns number of 'i' characters occuring in string s.

    >>> count_i('Mike Siff')
    2
    """
    a = 0
    for c in s:
        if c == 'i':
            a = a + 1
    return a


def count(s, x):
    """Returns number of occurences of character x in string s.

    >>> count('Mike Siff', 'i')
    2
    """
    a = 0
    for c in s:
        if c == x:
            a = a + 1
    return a


def remove_bangs(s):
    """Returns string like s except with all exclamations ('!') removed.

    >>> remove_bangs('That! Is! So! Cool!')
    'That Is So Cool'
    """
    t = ''
    for c in s:
        if c != '!':
            t = t + c
    return t


def remove_invisible(s):
    """Returns string like s but with invisble characters removed.

    >>> remove_invisible('Now is the time\\nfor all cool   stuff')
    'Nowisthetimeforallcoolstuff'
    """
    space_value = ord(' ')
    t = ''
    for c in s:
        if ord(c) > space_value:
            t = t + c
    return t


# eventually we will see a much easier way to do this
def last(s):
    """Returns the last character in s. Assumes s is not empty.

    >>> last('Whatever')
    'r'
    """
    for c in s:
        lc = c
    return lc


def largest_code(s):
    """
    Returns the largest code of any character in s. Assumes s is not empty.

    >>> largest_code('What?')   # largest code corresponds to 't'
    116
    """
    code = 0
    for c in s:
        if ord(c) > code:
            code = ord(c)
    return code


def nine_digits(m):
    """
    Returns string version of number m, up to 9 digits, padded with leading 0s
    if necessary. Assumes m is not more than 9 digits long.

    >>> nine_digits(37)
    '000000037'
    >>> nine_digits(65535)
    '000065535'
    """
    s = ''
    n = m
    for _ in range(9):
        r = n % 10
        d = ord('0') + r
        s = chr(d) + s
        n = n // 10
    return s


# --------------------------------------------------------------
# the examples below all use the simple_graphics module


def graphics_demo():
    """
    Demonstrates basic usage of simple_graphics module. Opens a graphics window
    and then draws an "X" using two diagonal "lines" of "paint" boxes. It then
    erases one of those diagonal lines. The graphics windows remains active
    when the function completes. User may then experiment further using paint
    and unpaint from the shell.
    """
    open_window()
    size = 10
    # draw diagonal line of length 10 boxes from upper left, down and right
    for i in range(size):
        paint(i, i)
    # draw diagonal line of length 10 boxes down and left
    for i in range(size):
        paint(size - i - 1, i)
    # erase first diagonal
    for i in range(size):
        unpaint(i, i)


def plot_squares(n):
    """
    Opens graphics window. Graphs the numbers 0 up to (but not including n)
    corresponding to its square.
    """
    open_window()
    for i in range(n):
        paint(i*i, i)


def plot_letters(s):
    """
    Opens graphics window. Assumes input string consists entirely of uppercase
    letters. For each letter, "graphs" a box in the row corresponding to its
    position in the string and in the column corresponding to its position in
    the alphabet.
    """
    open_window()
    y = 0
    for c in s:
        x = ord(c) - ord('A')
        if x >= 0:
            paint(x, y)
        y = y + 1
